#ifndef BLOCK_HH_
#define BLOCK_HH_
#include <iostream>
#include <fstream>
#include <string>
#include <vector>
#include "pvo-header.hh"

namespace PVO
{

  /** This controls the way blocks from existing survey files are
      dealt with. */
  namespace Block
  {
    const int headerLength = 640; ///< The length of the header string [bytes]
    /** The length of the data part of the block [bytes] */
    const int dataLength = 49152; // = 48*1024
    /** Four possible ways to express the equinox. These are return
	values for DataBlock::getEquinox().  */
    const std::string eq_date  = "Equat. of date"; 
    const std::string eq_b1950 = "B1950";
    const std::string eq_gal   = "GAL";
    const std::string eq_j2000 = "J2000";
    //   const std::string eq_equat = "J2000";
    const std::string eq_equat = "EQUAT";
    const float eq_1950 = 1950.;
    const float eq_2000 = 2000.;

    /**
     *  A single block of data.
     *
     *  This class holds one block of data: 48k long, plus 640 bytes
     *  of header information. It also records information about the 
     *  type of survey that the data is from, as well as the schema 
     *  defining how the headers are defined (which depends on the survey).
     */

    class DataBlock
    {
    public:
      /** Constructor for particular survey. */
      DataBlock(Header::SURVEY surveyType);
      virtual ~DataBlock(); ///< Destructor

      /** Read in a full block (headers+data) from a file stream. */
      bool readBlock(std::ifstream &file);

      /** Read in the headers from a file stream. */
      bool readHeader(std::ifstream &file);

      /** Read in the data from a file stream. */
      void readData(std::ifstream &file);

      /** Save an array of data to the Block. */
      void saveData(char *dataArray, int size);

      /** Get an individual data point. */
      char getDatum(int i){return data[i];};

      /** Get the entire string of data */
      char *getData(){return data;};

      /** Get the name of the survey */
      std::string getSurveyName(){return surveyName;};

      /** Get the type of survey */
      Header::SURVEY getSurveyType(){return surveyType;};

      /** Get the equinox in a string format. */
      std::string getEquinox();

      /** Get the coordinate type. */
      std::string getCoordMode();

      /** Was the telescope not moving, and tracking fine for this block? */
      bool telescopeOK();

      /** Generate the name of the fits file this block should be written to. */
      std::string getFITSname();

      /** Return a particular element of a keyword set */
      template <typename T> T getKeyword(int field, int elementNumber=0);

      /** Return the full set of values for a particular keyword */
      template <typename T> std::vector<T> getFullKeyword(int field);

      /** Provide a reference to the survey schema. */
      Header::SurveySchema& Survey(){
	Header::SurveySchema& rsurv=survey;
	return rsurv;
      };

      /** Provide a reference to the header string. */
      std::string& Header(){
	std::string& rhead=headerString;
	return rhead;
      };

    private:
      std::string headerString;    ///< the header string of the block
      char *data;                  ///< the set of data, stored as chars
      Header::SURVEY surveyType;   ///< what type of survey the block deals with
      Header::SurveySchema survey; ///< how the headers of the survey are defined
      std::string surveyName;      ///< the name of the survey
    };

  }

}

#endif /*BLOCK_HH_*/
