#include "pvo-header.hh"
#include <iostream>
#include <sstream>
#include <string>
#include <vector>

using std::string;

namespace PVO
{

  namespace Header
  {

    SurveySchema::SurveySchema(const SurveySchema& s)
    {
      for(int i=0;i<numKeywords;i++) startPosition[i]  = s.startPosition[i];
      for(int i=0;i<numKeywords;i++) fieldLength[i]    = s.fieldLength[i];
      for(int i=0;i<numKeywords;i++) fieldDimension[i] = s.fieldDimension[i];
      minSamples = s.minSamples;
      ant_x      = s.ant_x;
      ant_y      = s.ant_y;   
      ant_z      = s.ant_z;    
      frontend   = s.frontend;
      fd_poln    = s.fd_poln;  
      fd_hand    = s.fd_hand; 
      fd_sang    = s.fd_sang; 
      fd_xyph    = s.fd_xyph; 
      backend    = s.backend; 
      beconfig   = s.beconfig; 
      be_phase   = s.be_phase;
      be_dcc     = s.be_dcc;
      tcycle     = s.tcycle;   
      obs_mode   = s.obs_mode; 
      trk_mode   = s.trk_mode; 
      scanlen    = s.scanlen;  
      fd_mode    = s.fd_mode; 
      cal_mode   = s.cal_mode; 
      cal_freq   = s.cal_freq; 
      cal_dcyc   = s.cal_dcyc; 
      cal_phs    = s.cal_phs; 
      bmaj       = s.bmaj;
      bmin       = s.bmin;
      bpa        = s.bpa;
    }
    //-----------------------------------------------------------------------

    SurveySchema& SurveySchema::operator= (const SurveySchema& s)
    {
      if(this == &s) return *this;   // Gracefully handle self assignment
    
      for(int i=0;i<numKeywords;i++) this->startPosition[i]  = s.startPosition[i];
      for(int i=0;i<numKeywords;i++) this->fieldLength[i]    = s.fieldLength[i];
      for(int i=0;i<numKeywords;i++) this->fieldDimension[i] = s.fieldDimension[i];
      this->minSamples = s.minSamples;
      this->ant_x      = s.ant_x;
      this->ant_y      = s.ant_y;   
      this->ant_z      = s.ant_z;    
      this->frontend   = s.frontend;
      this->fd_poln    = s.fd_poln;  
      this->fd_hand    = s.fd_hand; 
      this->fd_sang    = s.fd_sang; 
      this->fd_xyph    = s.fd_xyph; 
      this->backend    = s.backend; 
      this->beconfig   = s.beconfig; 
      this->be_phase   = s.be_phase;
      this->be_dcc     = s.be_dcc;
      this->tcycle     = s.tcycle;   
      this->obs_mode   = s.obs_mode; 
      this->trk_mode   = s.trk_mode; 
      this->scanlen    = s.scanlen;  
      this->fd_mode    = s.fd_mode; 
      this->cal_mode   = s.cal_mode; 
      this->cal_freq   = s.cal_freq; 
      this->cal_dcyc   = s.cal_dcyc; 
      this->cal_phs    = s.cal_phs;  
      this->bmaj       = s.bmaj;
      this->bmin       = s.bmin;
      this->bpa        = s.bpa;

      return *this;
    }
    //--------------------------------------------------------------------------

    SurveySchema::SurveySchema(SURVEY surveyType)
    {
      /**
       * The constructor used to define a setup for a specific survey. The choice
       *  of survey determines the values of the start and length arrays for 
       *  each field. If a field does not appear in the header setup for a given
       *  survey, the start element is set to the RESERVE value, and the length
       *  is set to 0.
       * \param surveyType The type of survey, as expressed by the Header::SURVEY
       * enumeration.
       */

      // Generic PHU Keywords that are the same for all surveys with PKS.
      this->ant_x    = -4554231.5;
      this->ant_y    = 2816759.1;
      this->ant_z    = -3454036.3;
      this->backend  = "FB_1BIT";
      this->beconfig = "";
      this->be_phase = 0;  // 0=unknown
      this->be_dcc   = 0;
      this->tcycle   = 0;
      this->obs_mode = "SEARCH";
      this->trk_mode = "TRACK";
      this->cal_mode = "OFF"; 
      this->cal_freq = 0.;
      this->cal_dcyc = 0.;
      this->cal_phs  = 0.;
    
      if(surveyType == S70){
	//=============
	// 70-cm Survey
	//=============

	this->minSamples = 512 * 1024;

	this->frontend = "70cm_Cavity_Disk";
	this->fd_mode  = "FA";
	this->fd_poln  = "LIN";
	this->fd_hand  = 1;
	this->fd_sang  = 0.;
	this->fd_xyph  = 0.;

	this->bmaj = 0.75;
	this->bmin = 0.75;
	this->bpa = 0.;

	this->scanlen = 152.;

	// This initialises the SurveyHeader arrays for the S70 setup.
	int s70Start[numKeywords] = {0,8,12,16,20,
				     28,36,44,60,
				     72,74,90,106,114,
				     602,602,122,130,138,
				     202,204,220,228,
				     252,276,284,292,602,300,
				     304,320,332,348,364,372,380,388,
				     396,602,428,492,500,508,
				     516,598,520,524,536,540,
				     556,560,568,576,578,584,586,
				     588,602,602,602,602};
	for(int i=0;i<numKeywords;i++) this->startPosition[i] = s70Start[i];
      
	int s70Length[numKeywords] = {8,4,4,4,8,
				      8,8,16,12,
				      2,16,16,8,8,
				      0,0,8,8,64,
				      2,8,4,12,
				      12,4,8,8,0,4,
				      16,12,16,16,8,8,8,8,
				      8,0,8,8,8,8,
				      4,4,4,12,4,16,
				      2,8,4,2,6,2,2,
				      10,0,0,0,38};
	for(int i=0;i<numKeywords;i++) this->fieldLength[i] = s70Length[i];

	int s70Dim[numKeywords] = {1,1,1,1,1,
				   1,1,1,1,
				   1,1,1,1,1,
				   1,1,1,1,1,
				   1,2,2,2,
				   2,2,1,1,1,1,
				   1,1,1,1,1,1,1,1,
				   4,1,8,1,1,1,
				   1,1,1,1,1,1,
				   2,1,2,1,1,1,1,
				   1,1,1,1,1};
	for(int i=0;i<numKeywords;i++) this->fieldDimension[i] = s70Dim[i];
      }
      else if(surveyType == PM || surveyType == PH){
	//==================
	// Multibeam Surveys
	//==================

	this->minSamples = 2050 * 4096;

	this->frontend = "20cm_MB";
	this->fd_poln  = "LIN";
	this->fd_hand  = 1;
	this->fd_sang  = -90.;
	this->fd_xyph  = 0.;
	this->beconfig = "";
	this->fd_mode  = "CPA";
	//       this->fa_req  = 30.;

	this->bmaj = 0.24;
	this->bmin = 0.24;
	this->bpa = 0.;

	if(surveyType == PM) this->scanlen = 2100.;
	else                 this->scanlen = 265.;

	// This initialises the SurveyHeader arrays for the PM setup.
	int pmStart[numKeywords] = {0,6,10,16,20,
				    28,36,44,60,
				    72,74,90,106,114,
				    122,130,618,618,138,
				    202,204,220,228,
				    252,276,284,292,300,302,
				    304,320,332,348,364,372,380,388,
				    618,396,412,492,500,508,
				    618,516,520,524,618,540,
				    556,560,568,576,578,584,586,
				    588,598,606,614,618};
	for(int i=0;i<numKeywords;i++) this->startPosition[i] = pmStart[i];
      
	int pmLength[numKeywords] = {6,4,6,4,8,
				     8,8,16,12,
				     2,16,16,8,8,
				     8,8,0,0,64,
				     2,8,4,12,
				     12,4,8,8,2,1,
				     16,12,16,16,8,8,8,8,
				     0,4,4,8,8,8,
				     0,4,4,16,0,16,
				     2,8,4,2,6,2,2,
				     10,8,8,4,22};
	for(int i=0;i<numKeywords;i++) this->fieldLength[i] = pmLength[i];

	int pmDim[numKeywords] = {1,1,1,1,1,
				  1,1,1,1,
				  1,1,1,1,1,
				  1,1,1,1,1,
				  1,2,2,2,
				  2,2,1,1,1,2,
				  1,1,1,1,1,1,1,1,
				  1,4,20,1,1,1,
				  1,1,1,1,1,1,
				  2,1,2,1,1,1,1,
				  1,1,1,1,1};
	for(int i=0;i<numKeywords;i++) this->fieldDimension[i] = pmDim[i];
      }
      //     else throw xBadSurvey(surveyType);
    }
    //--------------------------------------------------------------------------

    template <class T> 
    T SurveySchema::readParameter(string headerString, int field,
				  int elementNumber)
    {
      /**
       * Read a parameter from a certain field in a header string. The function
       *  is templated, so one can obtain a numerical value or the string
       *  representation.
       * \param headerString The character string containing the full header.
       * \param field The number of the desired header field. Can use the 
       *  Header::FIELDS enumeration.
       * \param elementNumber Which element of a multi-valued field to return. 
       *  Defaults to 0 (ie. the first element).
       * \return The value of the parameter, in the format in which the function 
       *  is instantiated.
       */
      int dim = this->fieldDimension[field];
      int length = this->fieldLength[field];
      int pos = this->startPosition[field];
      std::stringstream ss;
      std::vector<T> output(dim);
      for(int i=0;i<dim;i++){
	ss.str( headerString.substr(pos+elementNumber*length, length) );
	ss >> output[i];    
      }
      return output[elementNumber];
    }
    template <> string 
    SurveySchema::readParameter<string>(string headerString, 
					int field, int elementNumber)
    {
      //    int dim = this->fieldDimension[field];
      int length = this->fieldLength[field];
      int pos = this->startPosition[field];
      return headerString.substr(pos+elementNumber*length, length);
    }
    template int SurveySchema::readParameter<int>(string headerString, 
						  int field,
						  int elementNumber);
    template float SurveySchema::readParameter<float>(string headerString, 
						      int field,
						      int elementNumber);
    template double SurveySchema::readParameter<double>(string headerString, 
							int field,
							int elementNumber);
    template string SurveySchema::readParameter<string>(string headerString, 
							int field,
							int elementNumber);
    //--------------------------------------------------------------------------



  }

}

