#ifndef HEADER_HH_
#define HEADER_HH_
#include <string>
#include <iostream>
#include <fstream>  

namespace PVO
{

  /** This controls how the headers of existing files are interpreted. */
  namespace Header
  {
    /** Codes for the different surveys.*/
    enum SURVEY { S70=0, ///< 70-cm Survey (Parkes Southern Pulsar Survey).
		  PM,    ///< Parkes Multibeam Survey
		  PH,    ///< Parkes High-Latitude (multibeam) Survey
		  DUD,   ///< Unknown survey -- this may be a problem!
		  NONEXISTENT ///< File does not exist.
    };

    /** Human-readable names for the surveys */
    const std::string surveyNames[DUD+1] = {
      "70-cm survey", 
      "Parkes Multibeam survey", 
      "Parkes High-latitude survey", 
      "Not a valid survey" 
    };

    /**
     *  What survey does a file contain?
     *
     *  Reads the first few bytes of a file, which contain information about
     *  the type of survey data recorded in the file.
     *  \param file String with name of the file to be examined.
     *  \return An enumerated Header::SURVEY type.
     */
    inline SURVEY idSurvey(std::string file)
    {
      std::ifstream fin;
      fin.open(file.c_str(), std::ios::binary);
      if(fin.fail()){
	fin.close();
	return NONEXISTENT;
      }
      char *hdr = new char[5];
      fin.read( hdr, 5*sizeof(char) );
      fin.close();
      hdr[4] = '\0';
      unsigned char compstring_s70[5] = {182, 'm', 'm', 219, 0};
      unsigned char compstring_pm[4] = {'P', 'M', 'D'};
      compstring_pm[3] = '\0';
      SURVEY returnval;
      if(strcmp((const char*)compstring_s70,hdr)==0) returnval = S70;
      else if(strncmp((const char*)compstring_pm,hdr,3)==0) returnval = PM;
      else returnval = DUD;
      delete [] hdr;
      return returnval;
    }

    /** Codes for the different moving states. \deprecated */
    enum ISMOVING { NOTGRID=0, POINT, MOVING, SCANNING };

    /** Codes representing the different header keywords */
    enum FIELDS 
      {CODE=0, VERSION, TAPENR, SCAN_CNTR, BLK_CNTR, 
       DATE, MJD, UT_STRT, LST_STRT, 
       COORD_TYPE, RA_STRT, DEC_STRT, GL_STRT, GB_STRT, 
       FEED_ANGLE, OBS_TIME, SCAN_RATE, SCAN_LEN, COMMENT, 
       NFILTER, CHANBW, NCHAN, FREQ_CHAN1, 
       SAMP_INT, SAMP_GRP, GRP_BLK, BLK_SEC, FDCNTRL, DATA_TYPE,
       UT, LST, RA, DEC, GL, GB, TEL_ZEN, TEL_AZ, 
       STATUS, ATTEN, TPOWER, NBLK_READ, SCAN_RATEL, SCAN_RATEB, 
       BYTE_GRP, NBEAM, IBEAM, PSRNAME, DATA_TWO, OBS_FILEN, 
       NBITS, DDM, NDDCH, MOVE, PNTERR, TREE, NSYS, 
       TELID, PANGLE, SMP_BSW, CAL_CYCLE, RESERVE
      };

    const int numKeywords = RESERVE + 1; ///< The total number of keywords.


    /**
     * Definitions of the different survey headers.
     *
     * This class contains the information needed to make sense of the header
     *  strings found in the existing survey data files. Each survey has a 
     *  different header structure, and so this class handles the definitions
     *  of where one keyword ends and another starts.
     */

    class SurveySchema
    {
    public:
      SurveySchema(){};
      virtual ~SurveySchema(){};
      SurveySchema(SURVEY surveyType); ///< Survey-specific constructor
      SurveySchema(const SurveySchema& s); ///< Copy constructor
      SurveySchema& operator= (const SurveySchema& s); ///< Copy constructor.

      /** Get the character number that a given field starts at. */
      int getStartPosition(int field){return startPosition[field];};
      /** Get the number of characters in a given field. */
      int getFieldLength(int field){return fieldLength[field];};
      /** Get the number of elements that a given field contains. */
      int getFieldDimension(int field){return fieldDimension[field];};

      template <class T> T readParameter(std::string headerString, int field,
					 int elementNumber = 0);
      ///< Read a particular header parameter.

      /** Exception class for a bad survey code. Not used at the moment. */
      class xBadSurvey{
      public:
	xBadSurvey(std::string s){surveyName=s;};
	virtual ~xBadSurvey(){};
	std::string getSurvey(){return surveyName;};
      private:
	std::string surveyName;
      };

      long        getMinSamples(){return minSamples;};

      // accessor functions for PHU keyword values -- only get functions needed.
      double      getAntX(){return ant_x;};
      double      getAntY(){return ant_y;};
      double      getAntZ(){return ant_z;};
      std::string getFrontend(){return frontend;};
      std::string getFd_poln(){return fd_poln;};
      int         getFd_hand(){return fd_hand;};
      float       getFd_sang(){return fd_sang;};
      float       getFd_xyph(){return fd_xyph;};
      std::string getBackend(){return backend;};
      std::string getBeconfig(){return beconfig;};
      int         getBe_phase(){return be_phase;};
      int         getBe_dcc(){return be_dcc;};
      double      getTcycle(){return tcycle;};
      std::string getObs_mode(){return obs_mode;};
      std::string getTrk_mode(){return trk_mode;};
      float       getScanlen(){return scanlen;};
      std::string getFd_mode(){return fd_mode;};
      std::string getCal_mode(){return cal_mode;};
      float       getCal_freq(){return cal_freq;};
      float       getCal_dcyc(){return cal_dcyc;};
      float       getCal_phs(){return cal_phs;};
      float       getBmaj(){return bmaj;};
      float       getBmin(){return bmin;};
      float       getBpa(){return bpa;};

 
    private:
      int startPosition[numKeywords];  ///< The character number at
      ///   which each field commences.
      int fieldLength[numKeywords];    ///< The number of characters in
      ///   each field.
      int fieldDimension[numKeywords]; ///< The number of elements in
      ///   each field.

      long        minSamples;  ///< The minimum number of samples to be
      ///   included in a FITS file

      // The following are PHU keywords that are survey-dependent.
      double      ant_x;
      double      ant_y;
      double      ant_z;
      std::string frontend;
      std::string fd_poln;
      int         fd_hand;
      float       fd_sang;
      float       fd_xyph;
      std::string backend;
      std::string beconfig;
      int         be_phase;
      int         be_dcc;
      double      tcycle;
      std::string obs_mode;
      std::string trk_mode;
      float       scanlen;
      std::string fd_mode;
      std::string cal_mode;
      float       cal_freq;
      float       cal_dcyc;
      float       cal_phs;

      float       bmaj;
      float       bmin;
      float       bpa;


    };

    // Inline functions

  }

}

#endif /*HEADER_HH_*/
